package com.changhong.epc.parsing.service.autosubmit.info.impl.cpn;

import com.changhong.epc.bean.count.format.FormCpnInfo;
import com.changhong.epc.parsing.service.autosubmit.info.Form;
import com.changhong.epc.parsing.service.autosubmit.info.FormCpn;
import com.changhong.epc.parsing.service.autosubmit.info.FormCpnListener;
import com.changhong.epc.parsing.service.autosubmit.info.FormulaInfo;
import com.changhong.epc.parsing.service.autosubmit.prop.Attribute;
import com.changhong.epc.parsing.service.autosubmit.prop.CpnMsg;
import com.changhong.epc.parsing.service.autosubmit.prop.MsgType;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;
import org.apache.commons.lang3.StringUtils;

import java.util.*;

/**
 * 普通控件抽象类
 * @author WangYX
 *
 */
@EqualsAndHashCode(callSuper=false, of={"fieldId", "index"})
public abstract class AbstractBaseFormCpn extends HashMap<Object, Object> implements FormCpn{

	/**
	 * 
	 */
	private static final long serialVersionUID = -1159764957350079218L;

	/**
	 * 映射key
	 */
	protected static final Map<Object, Object> MAPPING_KEY = Collections.singletonMap("值", Attribute.value);
	
	/* 表单控件监听 */
	protected FormCpnListener formCpnListener;
	
	/* 触发事件 */
	@Getter
	@Setter
	protected List<FormulaInfo> formulas;
	
	/* 控件别名 */
	@Getter
	@Setter
	protected String fieldId;
	
	/* 父控件 */
	@Setter
	protected FormCpn parentFormCpn;
	
	/* 在子表单中的下标 */
	@Getter
	@Setter
	protected Integer index;
	
	public AbstractBaseFormCpn(/*String fieldId*/) {
		/*this.fieldId = fieldId;*/
		this.formCpnListener = new BaseFormCpnListener();
	}
	
	public void init(FormCpnInfo formCpnInfo){
		this.fieldId = formCpnInfo.getTag_Attribute().getAlias();
	}
	
	@Override
	public Object get(Object key) {
		Object mappIngKey = MAPPING_KEY.get(key);
		Object val = super.get(mappIngKey == null ? key : mappIngKey);
		return val == null ? "" : val;
	}
	
	
	@Override
	public Object put(Object key, Object value) {
		if(StringUtils.isBlank(Objects.toString(key)))
			return null;
		if(Objects.equals(FORMULA_KEY, key))
			return setFormula((FormulaInfo) value);
		boolean isFormat = false;
		// 控件属性 可见，可编辑，值
		for(Attribute attr : Attribute.values()){
			if(Objects.equals(attr.toString(), key.toString())){
				key = Attribute.valueOf(Objects.toString(key));
				isFormat = true;
				break;
			}
		}

		if(!isFormat) {
			// 表单提示消息
			for (MsgType msg : MsgType.values()) {
				if (Objects.equals(msg.toString(), key.toString())) {
					((Form) get(FORM_KEY)).setMsg(MsgType.valueOf(Objects.toString(key)), Objects.toString(value, ""), this);
					return value;
				}
			}
			// 控件错误消息
			for (CpnMsg msg : CpnMsg.values()) {
				if (Objects.equals(msg.toString(), key.toString())) {
					key = CpnMsg.valueOf(Objects.toString(key));
					Object values = get(key);

					if(values instanceof List) {
						value = ((List) values).add(value);
					}else{
						value = Objects.toString(values);
					}

					break;
				}
			}
		}
		try{
			return super.put(key, value);
		}finally {
			if((key instanceof Attribute) && ((Attribute)key) == Attribute.value)
				formCpnListener.change(this);
		}
	}

	/*
	 * 添加事件
	 */
	public Object setFormula(FormulaInfo value) {
		if(this.formulas == null)
			this.formulas = new ArrayList<>();
		this.formulas.add(value);
		return value;
	}

	@Override
	public void setFormCpnListener(FormCpnListener formCpnListener) {
		this.formCpnListener = formCpnListener;
	}

	@Override
	public FormCpnListener getFormCpnListener() {
		return this.formCpnListener;
	}

	@Override
	public FormCpn getParentCpn() {
		return this.parentFormCpn;
	}

	public FormCpn clone(){
//		return FormCpnFactory.clone(this);
		return (FormCpn) super.clone();
	}

	@Override
	public Object getValue() {
		return get(Attribute.value);
	}
	
}
