package com.changhong.epc.parsing.service.loop.impl;

import com.changhong.epc.bean.admin.BatchInitCompany;
import com.changhong.epc.bean.admin.InitCompany;
import com.changhong.epc.bean.tenant.system.permission.PowerKey;
import com.changhong.epc.parsing.service.loop.exception.InitCompanyException;
import com.changhong.epc.rely.api.bean.Company;
import com.changhong.epc.rely.api.bean.Tenant;
import com.changhong.epc.rely.api.epc.admin.BatchInitCompanyApi;
import com.changhong.epc.rely.api.epc.admin.InitCompanyApi;
import com.changhong.epc.rely.api.epc.tenant.TenantInitApi;
import com.changhong.epc.rely.api.service.CscApiService;
import com.changhong.epc.rely.api.service.workFlow.WorkFlowService;
import com.changhong.epc.rely.api.tool.PowerTool;
import com.iemsoft.framework.cloud.core.thread.ThreadData;
import com.iemsoft.framework.cloud.core.tools.Assert;
import com.iemsoft.framework.cloud.core.tools.ObjectUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.math.NumberUtils;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Objects;
import java.util.Optional;

/**
 * 批量初始化租户公司服务
 * 操作状态：
 *  0:操作中、5:跳过(已经初始化)、8:操作失败、10:操作成功
 */
@Component
@Slf4j
public class InitCompanysLoopServiceImpl extends SimplAbstractLoopSuper<BatchInitCompany> {

    @Resource
    private BatchInitCompanyApi batchInitCompanyApi;

    @Resource
    private InitCompanyApi initCompanyApi;

    @Resource
    private CscApiService cscApiService;

    @Resource
    private WorkFlowService workFlowService;

    @Resource
    private TenantInitApi tenantInitApi;

    public static final String[] DATA_SOURCE_INFO = {"tenant_1", "tenant_2"};

    public static final int DATA_SOURCE_SIZE = DATA_SOURCE_INFO.length;

    static final String getDataSourceRef(Integer companyId){
        return DATA_SOURCE_INFO[(DATA_SOURCE_SIZE - 1) & companyId];
    }

    @Override
    public void init() {
        setLoopBaseService(batchInitCompanyApi);
        super.init();
    }

    @Override
    public String getThreadName() {
        return "批量初始化租户公司";
    }

    @Override
    void init(BatchInitCompany batchInitCompany) {
        super.init(batchInitCompany);
        ThreadData.set(OPEN_ID, batchInitCompany.getInsP());
        ThreadData.set(TENANT_ID, Objects.toString(batchInitCompany.getTenantId()));
        ThreadData.set(COMPANY_ID, batchInitCompany.getCompanyId());
    }

    @Override
    protected void doService() {
        businessHandle();
    }

    @Override
    void doService(BatchInitCompany batchInitCompany) {
        initCompanyApi.removeInte();
        if(ObjectUtil.notEmpty(initCompanyApi.getInitCompany(batchInitCompany.getTenantId(), batchInitCompany.getCompanyId()))){
            throw new InitCompanyException(5);
        }
        // 初始化云上行租户
        cscApiService.initTenant(batchInitCompany.getTenantId());

        InitCompany initCompany = getInitCompany(batchInitCompany);

        // 初始化工作流
        workFlowService.initialization(batchInitCompany.getTenantId(), batchInitCompany.getCompanyId());

        // 初始化租户数据
        tenantInitApi.init(initCompany);

        // 添加初始化公司记录
        // 初始化菜单数据
        initCompanyApi.initCompany(initCompany);
        initCompanyApi.removeInte();
    }

    @Override
    void success(BatchInitCompany batchInitCompany) {
        batchInitCompany.setDataStart(10);
        super.success(batchInitCompany);
    }

    @Override
    void error(BatchInitCompany batchInitCompany, Exception e) {
        if(e instanceof InitCompanyException){
            InitCompanyException initCompanyException = (InitCompanyException) e;
            batchInitCompany.setDataStart(initCompanyException.getDataState());
        }else{
            batchInitCompany.setDataStart(8);
        }
        super.error(batchInitCompany, e);
    }

    private InitCompany getInitCompany(BatchInitCompany batchInitCompany) {

        Integer tenantId = batchInitCompany.getTenantId()
                , companyId = batchInitCompany.getCompanyId();
        Tenant tenant = cscApiService.getTenantAndCompany(tenantId, companyId);
        Company company = Optional.ofNullable(tenant).map(Tenant::getCompanys).orElse(new ArrayList<>()).stream().findFirst().orElse(null);
        Assert.empty(tenant, "no this tenant:{0}", Objects.toString(tenantId));
        Assert.empty(company, "no this company:{0}", Objects.toString(companyId));

        InitCompany initCompany = new InitCompany();
        initCompany.setTenantId(tenantId);
        initCompany.setCompanyId(companyId);
        // 查询权限
        PowerKey powerKey = cscApiService.getSubCompanySystem(initCompany.getCompanyId());
        if(powerKey == null) {
            // 创建权限
            powerKey = PowerTool.createChildSystem(company, tenant);
        }
        initCompany.setPowerAppKey(powerKey.getAppKey());
        initCompany.setPowerSecretKey(powerKey.getSecretKey());
        initCompany.setInitState(0);
        initCompany.setDbName(getDataSourceRef(batchInitCompany.getCompanyId()));
        initCompany.setCompanyMasterId(Optional.ofNullable(company.getMasterCompanyId()).map(NumberUtils::createInteger).orElse(0));
        initCompany.setCompanyName(company.getName());
        initCompany.setTenantName(tenant.getName());
        initCompany.setCostTypeTempId(batchInitCompany.getCostTypeTempId());
        return initCompany;
    }

}
