package com.changhong.epc.parsing.service.loop.impl;

import com.changhong.epc.bean.parsing.FtpJson;
import com.changhong.epc.bean.parsing.FtpValues;
import com.changhong.epc.bean.tenant.master.MasterDefine;
import com.changhong.epc.bean.tenant.master.MasterElement;
import com.changhong.epc.bean.tenant.master.MasterValue;
import com.changhong.epc.bean.tenant.master.extend.GeneralMasterElement;
import com.changhong.epc.bean.tenant.master.extend.GeneralMasterValue;
import com.changhong.epc.bean.tenant.master.extend.MasterList;
import com.changhong.epc.bean.tenant.sunCloud.MasterFtp;
import com.changhong.epc.config.tool.MybatisTool;
import com.changhong.epc.constter.base.BaseConst;
import com.changhong.epc.constter.base.Context;
import com.changhong.epc.constter.exception.EPCServiceException;
import com.changhong.epc.parsing.service.cscApi.OrgApiService;
import com.changhong.epc.parsing.service.master.impl.MasterFtpServiceImpl;
import com.changhong.epc.rely.api.tool.VersionFilterApi;
import com.codingapi.tx.annotation.TxTransaction;
import com.fasterxml.jackson.core.type.TypeReference;
import com.iemsoft.framework.cloud.core.exception.IEMRuntimeException;
import com.iemsoft.framework.cloud.core.thread.Keys;
import com.iemsoft.framework.cloud.core.thread.ThreadData;
import com.iemsoft.framework.cloud.core.tools.FTPTool;
import com.iemsoft.framework.cloud.core.tools.JSONTool;
import com.iemsoft.framework.cloud.core.tools.ObjectUtil;
import com.iemsoft.framework.cloud.zuul.tool.RequestContextTool;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.*;

/**
 * 主数据同步服务
 * @ClassName: BillHandleLoopServiceImpl
 * @author [九鼎联合科技]
 * @date 2017年9月19日 下午6:52:07
 */
@Component("masterSynLoopServiceImpl")
@Slf4j
public class MasterSynLoopServiceImpl extends SimplAbstractLoopSuper<MasterFtp>{
	
	@Resource
	private MasterFtpServiceImpl masterFtpService;

	@Resource(name="orgApiServiceImpl")
	private OrgApiService cscApiService;

	@Value("${ftphost}")
	private String ftpHost;

	@Value("${ftpusername}")
	private String ftpUserName;

	@Value("${ftppassword}")
	private String ftpPassWord;

	private static final String PATH = "url";

	@Override
	public String getThreadName() {
		return "同步主数据服务";
	}

	@Override
	public void init() {
		super.setLoopBaseService(masterFtpService);
		super.init();
	}

	@Override
	protected void doService() {
		businessHandle();
	}

	@Override
	void doService(MasterFtp masterFtp) {
		startSyn(masterFtp);
	}
	protected void init(FtpJson ftj){
		//当前租户
		log.debug("当前主数据："+JSONTool.toJson(ftj));
		if(ftj.getValues() == null ||ftj.getValues().size()<=0){
			throw new IEMRuntimeException("主数据值为空");
		}
		FtpValues line = ftj.getValues().stream().findFirst().orElse(null);
		if(line == null){
		    return;
        }
		cscApiService.getCompanyAll(
		        line.getOrgCode2()
                , line.getOrgCode3()
                , t->{
                    log.debug("当前租户Id:{}", t.getId());
                    ThreadData.set(Keys.TENANT_ID, Objects.toString(t.getId()));
                    ThreadData.set(BaseConst.TENANT_ID, Objects.toString(t.getId()));
                }
                , c->{
                    log.debug("当前公司Id:{}", c.getId());
                    ThreadData.set(Keys.COMPANY_ID, Objects.toString(c.getId()));
                    ThreadData.set(BaseConst.COMPANY_ID, c.getId());
                }
        );
		MybatisTool.initDataSource();
	}
	/**
	 * 开始同步主数据
	 */
//	@Transactional
	public void startSyn(MasterFtp ftp) {
		List<Map> list = JSONTool.toList(ftp.getFilePaths(), Map.class);
		if(ObjectUtil.empty(list)){
			return;
		}
		//循环ftp路径
		for (Map<String, String> ma : list) {
			//ftp路径
			String path = ma.get(PATH);
			String b = path.substring(path.lastIndexOf("/") + 1, path.indexOf(".txt"));
			String[] as = b.split("-");
			//主数据定义
			String defineCode = as[0];
			FtpJson ftj = JSONTool.toObj(getFtpJson(path).toString(), FtpJson.class);
			//两次判断， 一次 判断通用还是租户
			switch (ftp.getDataType()) {
				case "1":
					//租户主数据
					init(ftj);
					synTenantMaster(defineCode,ftj);
					break;
				case "2":
					//通用主数据
					synurrencyMaster(defineCode,ftj);
					break;
				case "3":
					//其他
					break;
				default:
					throw new EPCServiceException("订阅类型不存在");
			}
		}
	}
	/**
	 * 租户主数据同步
	 */
	@TxTransaction(isStart = true)
	@Transactional
	public void synTenantMaster(String code,FtpJson json){
		List<MasterElement> eleList = new ArrayList<>();
		List<MasterValue> valList = new ArrayList<>();
		//check定义
		MasterDefine ma = new MasterDefine();
		ma.setCode(code);
		ma.setName(json.getHeads().getName());
		ma.setType(json.getHeads().getType());
		ma.setDelFlag(0);
		ma.setDataStart(0);
		ma.setInsT(new Date());
		ma.setTenantId(BigInteger.valueOf(Long.parseLong(ThreadData.get(Keys.TENANT_ID))));
		log.debug("开始check主数据定义:"+code);
		//更新版本
		VersionFilterApi.filter(ma.getCode(), Context.getTenantIdStr(), Context.getCompanyIdStr());
		masterFtpService.checkMasterDefine(ma);
		//元素集合
        if(json.getHeads()!=null && json.getHeads().getEles()!=null && json.getHeads().getEles().size()>0) {
            json.getHeads().getEles().stream().forEach(o -> {
                MasterElement gen = new MasterElement();
                gen.setDefineCode(code);
                gen.setCode(o.getCode());
                gen.setName(o.getName());
                gen.setType(o.getType());
                gen.setTenantId(BigInteger.valueOf(Long.parseLong(ThreadData.get(Keys.TENANT_ID))));
                gen.setDelFlag(0);
                gen.setDataStart(0);
                gen.setInsT(new Date());
                eleList.add(gen);
            });
        }
		//check元素
		if(eleList.size()>0) {
			log.debug("开始check主数据元素");
			masterFtpService.checkMasterElement(eleList, code);
		}
		//value集合
        if(json.getValues()!=null && json.getValues().size()>0) {
            json.getValues().stream().forEach(o -> {
                if (o.getOperType() == 3) {
                    MasterValue mv = new MasterValue();
                    mv.setDefineCode(code);
                    mv.setFlag(o.getOperType().toString());
                    mv.setTenantId(ThreadData.get(Keys.TENANT_ID));
                    mv.setRowNo(o.getRowId());
                    mv.setInsT(new Date());
                    valList.add(mv);
                } else {
                    o.getData().stream().forEach(c -> {
                        MasterValue g = new MasterValue();
                        g.setDefineCode(code);
                        g.setRowNo(o.getRowId());
                        g.setEleCode(c.getName());
                        g.setEleValue(c.getValue());
                        g.setFlag(o.getOperType().toString());
                        g.setTenantId(ThreadData.get(Keys.TENANT_ID));
                        g.setDelFlag(0);
                        g.setDataStart(0);
                        g.setInsT(new Date());
                        valList.add(g);
                    });
                }

            });
        }
		//添加主数据值
		if(valList.size()>0) {
			log.debug("开始check主数据值");
			masterFtpService.checkMasterValue(valList);
		}

	}
	
	/**
	 * 通用主数据同步
	 * @throws Exception 
	 */
	@SuppressWarnings("unused")
	@Transactional
	public void synurrencyMaster(String code,FtpJson json){
		//准备参数
		List<GeneralMasterElement> eleList = new ArrayList<>();
		List<GeneralMasterValue> valList = new ArrayList<>();
		MasterList list = new MasterList();
		list.setName(json.getHeads().getName());
		list.setType(json.getHeads().getType());
		list.setEleList(eleList);
		list.setValList(valList);
		list.setCode(code);
		//元素集合
		if(json.getHeads()!=null && json.getHeads().getEles()!=null && json.getHeads().getEles().size()>0) {
			json.getHeads().getEles().stream().forEach(o->{
				GeneralMasterElement g = new GeneralMasterElement();
				g.setDefineCode(code);
				g.setCode(o.getCode());
				g.setName(o.getName());
				g.setType(o.getType());
				eleList.add(g);
			});
		}
		//主数据值集合
		if(json.getValues()!=null && json.getValues().size()>0) {
			json.getValues().stream().forEach(o->{
				o.getData().stream().forEach(c->{
					GeneralMasterValue g = new GeneralMasterValue();
					g.setDefineCode(code);
					g.setRowNo(o.getRowId());
					g.setEleCode(c.getName());
					g.setEleValue(c.getValue());
					g.setFlag(o.getOperType().toString());
					valList.add(g);
				});
				masterFtpService.synMaster(list);
				valList.clear();
			});
		}
        //更新版本
        String tenandId=list.getTenantId()==null?"0":list.getTenantId().toString();
        VersionFilterApi.filter(list.getCode(), Context.getTenantIdStr(), Context.getCompanyIdStr());
//		Integer count = masterFtpService.synMaster(list);
//		if(count<0){
//			throw new EPCServiceException("通用主数据添加异常");
//		}
		/*//主数据redis更新版本号*/
        VersionFilterApi.filter(code, null, null);
	}
	
	//ftp请求
	public Object getFtpJson(String path){
		log.debug("当前ftp路径:"+path);
		Object a = FTPTool.getJSONFile(new FTPTool.FTP(ftpHost, ftpUserName, ftpPassWord)
				,path, new TypeReference<String>(){}.getType());
		log.debug("订阅ftp返回值:"+a);
		log.debug("订阅ftp返回值:"+JSONTool.toJson(a));
		return a;
	}

}
