package com.changhong.epc.parsing.service.transition.billInfo;

import com.alibaba.fastjson.JSON;
import com.changhong.epc.bean.parsing.BillInfo;
import com.changhong.epc.bean.tenant.bill.IOCRBill;
import com.changhong.epc.bean.tenant.bill.OCRBillBodyEntity;
import com.changhong.epc.bean.tenant.bill.OCRBillEntity;
import com.changhong.epc.bean.tenant.bill.OCRBillHeaderEntity;
import com.changhong.epc.config.tool.ContextEnvironment;
import com.changhong.epc.constter.system.SystemConst;
import com.changhong.epc.parsing.service.bill.IAutoAllotJourney;
import com.changhong.epc.parsing.service.bill.impl.AutoAllotJourneyImpl;
import com.changhong.epc.parsing.service.bill.impl.OtherTypeJourneyImpl;
import com.changhong.epc.parsing.service.transition.billInfo.formula.ExpressionOarser;
import com.changhong.epc.parsing.service.transition.billInfo.models.TransverterBillInfoPre;
import com.googlecode.aviator.AviatorEvaluator;
import com.googlecode.aviator.runtime.function.AbstractFunction;
import com.googlecode.aviator.runtime.type.AviatorObject;
import com.googlecode.aviator.runtime.type.AviatorString;
import com.iemsoft.framework.cloud.core.tools.ObjectUtil;
import com.iemsoft.framework.cloud.core.tools.SpringUtil;
import com.iemsoft.framework.cloud.core.tools.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.math.NumberUtils;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;
import org.springframework.ui.ModelMap;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * 差旅报销费用类型解析票据
 */
@Slf4j
@Component("transverterBillInfo")
@Scope("prototype")
public class TravelTypeTransverterBillInfo extends AbstractTransverterBillInfo implements SystemConst {

	/**
	 * 获得值code
	 * @param mdCode
	 * @param val
	 * @return
	 */
	private String getValCode(String mdCode, String val, boolean ignore){
		return gainValCodeTranche.valToCode(mdCode, val, ignore);
	}
	private String getValCode(String mdCode, String val){
		return gainValCodeTranche.valToCode(mdCode, val, false);
	}

	/**
	 * 执行转换票据
	 * @param oCRBillEntity
	 */
	public boolean exeConvert(OCRBillEntity oCRBillEntity){
		init();
		// 一： 准备数据
		// 1.1 查询出原始Billinfo 数据
		for (OCRBillHeaderEntity billHead : oCRBillEntity.getBillDatas()) {
			// billBodyList
			List<OCRBillBodyEntity> billBodyList = billHead.getBillBodyEntityList();
			// 模板code
			String billModelCode = billHead.getBillModelCode();

			// 1.2 查询票据所有模板
			List<List<Map<String,String>>> findMasterValueByOMCode = findMasterValueByOMCode(billModelCode, oCRBillEntity.getFormId());

			setBillInfo(billHead, findMasterValueByOMCode);

			if(ObjectUtil.notEmpty(billBodyList)){
				billBodyList.stream().forEach(billBody->setBillInfo(billBody, findMasterValueByOMCode));
			}

		}	
		
		return gainValCodeTranche.isLoseMaster();
	}

	@Override
	public IAutoAllotJourney getAutoAllotJourney(BillInfo billInfo) {
		return new ContextEnvironment().travelType(
				billInfo.getCtId()
				, ()->SpringUtil.getBean(AutoAllotJourneyImpl.class)
				, ()->SpringUtil.getBean(AutoAllotJourneyImpl.class)
				, ()->SpringUtil.getBean(OtherTypeJourneyImpl.class)
		);
	}

	/**
	 * 添加表单信息
	 * @param iocrBill
	 * @param findMasterValueByOMCode
	 */
	public void setBillInfo(IOCRBill iocrBill, List<List<Map<String,String>>> findMasterValueByOMCode){
		// 票据元素信息
		Map<String, String> billOriginalInfo = iocrBill.getBillOriginalInfo();

		// 解析票据取值表
		for (Map<String, String> map : findMasterValueByOMCode.get(1)) {

			Object object = getVal(map.get(BILL_FATC_VAL_ELEMENT), billOriginalInfo);
			// 费用类型
			if(Objects.equals(costTypeMdCode, map.get(BILL_FATC_VAL_MDCODE))){
				iocrBill.setCostType(new IOCRBill.CostType(
						// 查询code
						getValCode(map.get(BILL_FATC_VAL_MDCODE),map.get(BILL_FATC_VAL_ELEMENT))
						// 费用类型名称
						, map.get(BILL_FATC_VAL_ELEMENT)
				));
			}
			// 票据元素

			if(object != null){
				// 主数据code
				String mdCode = map.get(BILL_FATC_VAL_MDCODE);
				// 票据别名
				String billKey = map.get(BILL_FATC_VAL_KEY);
				// 票据值
				String val = format(billKey, StringUtil.toString(object));

				setBillInfo(iocrBill, mdCode, billKey, val, null);
			}


		}
		// 解析票据取值公式表
		for (Map<String, String> map : findMasterValueByOMCode.get(0)) {

			TransverterBillInfoPre pre = new TransverterBillInfoPre();
			pre.setCondition(map.get(BILL_FATC_COUNT));
			pre.setElementList(map.get(BILL_FATC_ELEMENT));
			pre.setMatchingVal(billOriginalInfo);
			ExpressionOarser expressionOarser = new ExpressionOarser(pre);

			if(expressionOarser.execute()){
				if(Objects.equals(costTypeMdCode, map.get(BILL_FATC_MDCODE))){
					iocrBill.setCostType(new IOCRBill.CostType(
							// 查询code
							map.get(BILL_FATC_VALUE_CODE)
							// 费用类型名称
							, map.get(BILL_FATC_VALUE)
					));
				}
				else{

					// 主数据code
					String mdCode = map.get(BILL_FATC_MDCODE);
					// 票据别名
					String billKey = map.get(BILL_FATC_MEANIG_KEY);
					// 票据值
					String val = format(billKey, StringUtil.toString(map.get(BILL_FATC_VALUE)));

					setBillInfo(iocrBill, mdCode, billKey, val, Objects.toString(map.get(BILL_FATC_VALUE_CODE), ""));
				}

			}
		}

	}

	protected void setBillInfo(IOCRBill iocrBill, String mdCode, String billKey, String val, String code){
		if(ObjectUtil.notEmpty(mdCode)) {
			// 票据值转code
			if(ObjectUtil.empty(code)) {
				code = getValCode(mdCode, val, Objects.equals(billKey, PERSONAL_NAME));
			}
			iocrBill.setBillMasterName(StringUtil.trim(mdCode), val);
			iocrBill.setBillMasterCode(StringUtil.trim(mdCode), code);
			iocrBill.setBillKeyName(StringUtil.trim(billKey), val);
			iocrBill.setBillKeyCode(StringUtil.trim(billKey), code);
		}else {
			iocrBill.setBillKeyName(StringUtil.trim(billKey), val);
			iocrBill.setBillKeyCode(StringUtil.trim(billKey), val);
		}
	}

	public static void main(String... args){
		System.out.println(
				getVal("{fun}add(add('1001', '1002'), '1003')", new HashMap<String, String>(){
					{
						put("1001", "100.3");
						put("1002", "40.399");
						put("1003", "1");
					}
				})
		);
	}

	protected static String getVal(String express, Map<String, String> billOriginalInfo){
		if(express.indexOf("{fun}") == 0){
			return Objects.toString(
					AviatorEvaluator.execute(express.substring("{fun}".length(), express.length()), new ModelMap().addAllAttributes(billOriginalInfo))
			);
		}
		return billOriginalInfo.get(express);
	}

	static {
		AviatorEvaluator.addFunction(new AbstractFunction() {
			@Override
			public String getName() {
				return "add";
			}

			@Override
			public AviatorObject call(Map<String, Object> env, AviatorObject arg1, AviatorObject arg2) {
				Object val1 = getObj(env, arg1);
				Object val2 = getObj(env, arg2);
				return add(val1, val2);
			}

			public Object getObj(Map<String, Object> env, AviatorObject arg){
				if(arg instanceof AviatorStringRes){
					return arg.stringValue(env);
				}else{
					return env.get(arg.stringValue(env));
				}
			}

			public AviatorObject add(Object val1, Object val2){
				if(NumberUtils.isNumber(Objects.toString(val1))
						&& NumberUtils.isNumber(Objects.toString(val2))){
					return new AviatorStringRes(
							/*toNumber(NumberUtils.createDouble(Objects.toString(val1))
									+ NumberUtils.createDouble(Objects.toString(val2))
							)*/
							toNumber(new BigDecimal(Objects.toString(val1)).add(new BigDecimal(Objects.toString(val2))).doubleValue())
					);
				}
				return new AviatorStringRes(
						Objects.toString(val1) + Objects.toString(val2));
			}

		});
	}

	public static String toNumber(double d) {
		NumberFormat nf = new DecimalFormat("0.00");

		// 如果不需要四舍五入，可以使用RoundingMode.DOWN
		nf.setRoundingMode(RoundingMode.UP);

		return nf.format(d);
	}

	public static class AviatorStringRes extends AviatorString{
		public AviatorStringRes(String lexeme) {
			super(lexeme);
		}
	}

}

